"""
This module provides utilities for working with pip and setup.cfg files.

Functions:
- read_setup_config: Read the setup.cfg file in the project directory.
- install_packages_from_options: Install packages from the options in setup.cfg.
- install_requires: Install packages from the install_requires option in setup.cfg.
- tests_require: Install packages from the tests_require option in setup.cfg.
- build_dependency_wheels: Build dependency wheels for the project.

"""

import configparser
import pip

from isee.common import get_env_var, get_file_path


def read_setup_config(project_dir=None):
    if not project_dir:
        project_dir = get_env_var("GITHUB_WORKSPACE")
    path = get_file_path("setup.cfg", project_dir)
    config = configparser.ConfigParser()
    config.read(path)
    return config


def install_packages_from_options(config_options, key):
    if _p := config_options.get(key):
        pkgs = [x for x in _p.split("\n") if x]
        pip.main(["install"] + pkgs)
    else:
        print(f"No {key} packages to install")


def install_requires(*, project_dir=None):
    config = read_setup_config(project_dir)
    install_packages_from_options(config["options"], "install_requires")


def tests_require(*, project_dir=None):
    """Install from tests_require in setup.cfg options"""
    config = read_setup_config(project_dir)
    install_packages_from_options(config["options"], "tests_require")


def build_dependency_wheels(repository_dir, wheelhouse, requirements_filepath=None):
    args = ["wheel", "--wheel-dir", wheelhouse, "--find-links", wheelhouse]
    if requirements_filepath:
        args.extend(["--requirement", requirements_filepath])
    args.extend(["--editable", repository_dir])
    pip.main(args)



"""
This module contains common functions that are used in multiple modules.

Functions:
- git: Execute a git command and return the output.
- get_env_var: Get the value of an environment variable.
- get_file_path: Get the file path of a file in a directory.

"""

import subprocess
import os
import glob
from wads.util import git as wads_git


def git(*args, work_tree=".", git_dir=None):
    """
    Execute a git command and return the output.

    >>> git('status')  # doctest: +SKIP
    On branch master
    Your branch is up to date with 'origin/master'.
    <BLANKLINE>
    nothing to commit, working tree clean

    """

    try:
        # return subprocess.check_output(['git'] + list(args)).decode().strip()  # OLD
        return wads_git(" ".join(args), work_tree=work_tree, git_dir=git_dir)
    except subprocess.CalledProcessError as e:
        # Print the error message and return the output
        print(f"Error executing git command: {e}")
        print(f"Standard output: {e.output.decode().strip()}")
        print(f"Exit code: {e.returncode}")
        return e.output.decode().strip()


def get_env_var(key):
    """
    Get the value of an environment variable.
    If the variable is not defined or is empty, raise a RuntimeError.

    :param key: The name of the environment variable.
    :return: The value of the environment variable.

    :raise RuntimeError: If the environment variable is not defined or is empty.

    >>> import os
    >>> os.environ['TEST'] = 'test'
    >>> get_env_var('TEST')
    'test'

    >>> get_env_var('TEST2')
    Traceback (most recent call last):
    ...
    RuntimeError: TEST2 is not defined or is empty!

    >>> os.environ['TEST3'] = ''
    >>> get_env_var('TEST3')
    Traceback (most recent call last):
    ...
    RuntimeError: TEST3 is not defined or is empty!
    """
    value = os.getenv(key)
    if not value:
        raise RuntimeError(f"{key} is not defined or is empty!")
    return value


def get_file_path(filename, root_path):
    result = glob.glob(root_path + f"/**/{filename}", recursive=True)
    if len(result) == 0:
        raise RuntimeError(
            f'No file with name "{filename}" exist into the directory "{root_path}"!'
        )
    if len(result) > 1:
        raise RuntimeError(
            f'More than one file with name "{filename}" exist into the directory "{root_path}"!'
        )
    return result[0]



"""
This module provides utilities for updating files.

Functions:
- update_helm_tpl: Update the Helm chart template files.
- update_manifest: Update the manifest file.
- update_setup_cfg: Update the setup.cfg file.
- update_setup_py: Update the setup.py file.
- _get_setup_filepath: Get the setup file path.
- _update_file: Update the file content.

"""

import re
import os

from isee.common import get_env_var, get_file_path


def update_helm_tpl():
    def update_helpers_tpl(root_path):
        hostname = get_env_var("AWS_HOSTNAME")
        repository = get_env_var("AWS_REPOSITORY")
        image_version = get_env_var("IMAGE_VERSION")
        path = get_file_path("_helpers.tpl", root_path)
        pattern = rf'({{{{- define "{repository}.image" }}}}{hostname}\/{repository}:).+({{{{- end -}}}})'
        _update_file(path, pattern, rf"\g<1>{image_version}\g<2>")

    def update_chart_config(root_path):
        chart_version = get_env_var("CHART_VERSION")
        path = get_file_path("Chart.yaml", root_path)
        _update_file(path, r"version: [\d.]+", f"version: {chart_version}")

    root_path = get_env_var("HELM_TPL_DIR")
    update_helpers_tpl(root_path)
    update_chart_config(root_path)


def update_manifest(manifest_path: str):
    repository = get_env_var("AWS_REPOSITORY")
    chart_version = get_env_var("CHART_VERSION")
    pattern = rf'("chartName":"adi\/{repository}",(\n\s*)?"chartVersion":")[\d.]+'
    _update_file(manifest_path, pattern, rf"\g<1>{chart_version}")


def update_setup_cfg(*, project_dir=None, version=None):
    path = _get_setup_filepath("setup.cfg", project_dir)
    version = version or get_env_var("VERSION")
    _update_file(path, r"version\s=\s.+", f"version = {version}")


def update_setup_py(*, project_dir=None, version=None):
    path = _get_setup_filepath("setup.py", project_dir)
    version = version or get_env_var("VERSION")
    _update_file(path, r"version='.+',", f"version='{version}',")


def _get_setup_filepath(filename, project_dir):
    project_dir = project_dir or get_env_var("GITHUB_WORKSPACE")
    return os.path.join(project_dir, filename)


def _update_file(path, pattern, replace, content_must_change=False):
    with open(path, "r+") as file:
        content = file.read()
        content_new = re.sub(pattern, replace, content, flags=re.M)
        if content_new == content and content_must_change:
            raise RuntimeError(
                f'File content unchanged. Failed to update file "{path}"!'
            )
        file.seek(0)
        file.write(content_new)
        file.truncate()
